const Generator = require('yeoman-generator')
const yosay = require('yosay')

const { getPackages } = require('../packages')
const { projectTypes } = require('../utils')

module.exports = class LionByte extends Generator {
  prompting() {
    this.log(yosay(`Welcome to the best ${'generator-lionbyte'} generator!`))

    return this.prompt([
      {
        type: 'input',
        name: 'name',
        message: 'Your project name',
        'default': this.appname.replace(/ /g, '-').toLowerCase()
      },
      {
        type: 'input',
        name: 'description',
        message: 'Project description',
        'default': ''
      },
      {
        type: 'list',
        name: 'projectType',
        message: 'What type of project are you doing?',
        choices: projectTypes,
        'default': 'node'
      },
      {
        type: 'confirm',
        name: 'gitHooks',
        message: 'Do you want to add linting to your pre-commit hooks?',
        'default': false
      }
    ]).then(props => {
      this.config.set('name', props.name.replace(/ /g, '-').toLowerCase())
      this.config.set('description', props.description)
      this.config.set('projectType', props.projectType)
      this.config.set('gitHooks', props.gitHooks)

      const subGenerators = [
        '../common',
        `../${props.projectType}`,
        '../package',
        '../typescript'
      ]

      subGenerators.forEach(sub => {
        this.composeWith(require.resolve(sub), {})
      })
    })
  }

  async install() {
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    // @ts-ignore
    const { devDependencies, dependencies } = getPackages(this.config.getAll())

    if (dependencies.length > 0) {
      this.npmInstall(dependencies)
    }

    this.npmInstall(devDependencies, { saveDev: true })
  }
}
